using System;
using StudioX.Domain.Entities.Auditing;
using StudioX.Timing;

namespace StudioX.Application.Services.Dto
{
    /// <summary>
    ///     A shortcut of <see cref="CreationAuditedEntityDto" /> for most used primary key type (<see cref="int" />).
    /// </summary>
    [Serializable]
    public abstract class CreationAuditedEntityDto : CreationAuditedEntityDto<int>
    {
    }

    /// <summary>
    ///     This class can be inherited for simple Dto objects those are used for entities implement
    ///     <see cref="ICreationAudited" /> interface.
    /// </summary>
    /// <typeparam name="TPrimaryKey">Type of primary key</typeparam>
    [Serializable]
    public abstract class CreationAuditedEntityDto<TPrimaryKey> : EntityDto<TPrimaryKey>, ICreationAudited
    {
        /// <summary>
        ///     Creation date of this entity.
        /// </summary>
        public DateTime CreationTime { get; set; }

        /// <summary>
        ///     Creator user's id for this entity.
        /// </summary>
        public long? CreatorUserId { get; set; }

        /// <summary>
        ///     Constructor.
        /// </summary>
        protected CreationAuditedEntityDto()
        {
            CreationTime = Clock.Now;
        }
    }
}